vim.o.completeopt = 'menu,menuone,noselect,preview'
vim.g.completion_enable_auto_popup = 1

local cmp = require('cmp')
local compare = require('cmp.config.compare')
local luasnip = require('luasnip')

cmp.setup{
	completion = {
		keyword_length = 0,
		autocomplete = false,
	},

	view = {
		entries = {
			name = 'custom',
			selection_order = 'near_cursor',
		},
	},

	window = {
		completion = {
			winhighlight = 'NormalFloat:TelescopeNormal,FloatBorder:TelescopeBorder',
			col_offset = 1,
			side_padding = 0,
		},

		documentation = {
			winhighlight = 'NormalFloat:TelescopeNormal,FloatBorder:TelescopeBorder',
			side_padding = 0,
		}
	},

	formatting = {
		fields = { 'abbr', 'menu' },
		format = function(entry, vim_item)
			if #vim_item.abbr > 25 then
				vim_item.abbr = string.sub(vim_item.abbr, 1, 23) .. '...'
			end
			return vim_item
		end
	},

	snippet = {
		expand = function(args)
			luasnip.lsp_expand(args.body)
		end,
	},

	mapping = {
		['<C-j>'] = cmp.mapping.select_next_item{},
		['<C-k>'] = cmp.mapping.select_prev_item{},
		['<Tab>'] = cmp.mapping(cmp.mapping.scroll_docs(-4), { 'i', 'c' }),
		['<S-Tab>'] = cmp.mapping(cmp.mapping.scroll_docs(4), { 'i', 'c' }),
		['<C-l>'] = cmp.mapping(cmp.mapping.complete(), { 'i', 'c' }),
		['<CR>'] = cmp.mapping.confirm{
			behavior = cmp.ConfirmBehavior.Insert,
			select = true
		},
	},

	sources = cmp.config.sources({
		{ name = 'nvim_lua' },
		{ name = 'nvim_lsp' },
		{ name = 'luasnip' },
		{
			name = 'path',
			option = {
				get_cwd = function()
					return vim.fn.getcwd()
				end,
			},
		},
		{ name = 'nerdfont', keyword_length = 3, },
	}),

	sorting = {
		priority_weight = 1,
		comparators = {
			compare.offset,
			compare.exact,
			compare.score,
			compare.recently_used,
			compare.kind,
			compare.sort_text,
			compare.length,
			compare.order,
		},
	},

	experimental = {
		ghost_text = true,
	},
}

cmp.setup.cmdline({':'}, {
	mapping = cmp.mapping.preset.cmdline(),
	sources = cmp.config.sources({
		{ name = 'path' }
	}, {
		{ name = 'cmdline' }
	}),
	matching = { disallow_symbol_nonprefix_matching = false },
})

vim.keymap.set({'i', 's'}, '<C-s>', function ()
	if luasnip.expand_or_jumpable() then
		luasnip.expand_or_jump()
	end
end, { silent = false })

vim.keymap.set('i', '<C-a>', function ()
	if luasnip.choice_active() then
		luasnip.change_choice(1)
	end
end, { silent = false })

local fmt = require('luasnip.extras.fmt').fmt
local s = luasnip.s
local sn = luasnip.sn
local i = luasnip.insert_node
local t = luasnip.text_node
local c = luasnip.choice_node
local d = luasnip.dynamic_node
local f = luasnip.function_node
local rep = require('luasnip.extras').rep

local ols_json =
s('ols', fmt([[{{
	"$schema": "https://raw.githubusercontent.com/DanielGavin/ols/master/misc/ols.schema.json",
	"enable_format": {},
	"enable_semantic_tokens": true,
	"enable_document_symbols": true,
	"enable_hover": true,
	"enable_snippets": true,
	"enable_inlay_hints": true,
	"enable_fake_methods": true,
	"profile": "default",
	"profiles": [
		{{ "name": "default", "checker_path": [{}]}}
	],
	"checker_args": "--no-threaded-checker"
}}]], { c(2, { t 'false', t 'true' }), i(1) }))

local odinfmt_json =
luasnip.parser.parse_snippet('odinfmt', [[{
	"$schema": "https://raw.githubusercontent.com/DanielGavin/ols/master/misc/odinfmt.schema.json",
	"character_width": 100,
	"tabs": true,
	"tabs_width": 8,
	"sort_imports": true
}]])

local tasks_json =
s('tasks', fmt([[{{
	"version": "2.0.0",
	"tasks": [
		{{
			"label": "{}",
			"type": "shell",
			"command": "{}",
			"options": {{
				"cwd": "${{workspaceFolder}}"
			}}
		}}
	]
}}]], { i(1), i(2) }))

luasnip.add_snippets(nil, {
	all = {},
	json = {
		ols_json,
		odinfmt_json,
		tasks_json,
	},
	odin = {
		s('im', fmt('import {}"{}"', { i(0), i(1) })),
		s('ba', fmt('import {}"base:{}"', { i(0), i(1) })),
		s('co', fmt('import {}"core:{}"', { i(0), i(1) })),
		s('vd', fmt('import {}"vendor:{}"', { i(0), i(1) })),
		s('sh', fmt('import {}"shared:{}"', { i(0), i(1) })),
		s('proc', fmt('{} :: proc({}) {}{{\n\t{}\n}}', { i(1, 'name'), i(2), i(3), i(0) })),
		s('struct', fmt('{} :: struct{{\n\t{}\n}}', { i(1, 'Name'), i(2) })),
		s('union', fmt('{} :: union{{\n\t{}\n}}', { i(1, 'Name'), i(2) })),
		s('enum', fmt('{} :: enum{{\n\t{}\n}}', { i(1, 'Name'), i(2) })),
		s('flags', fmt('{}_Flag :: enum{{\n\t{}\n}}\n{}_Flags :: bit_set[{}_Flag]', { i(1, 'Name'), i(2), rep(1), rep(1) })),
		s('md', fmt('{} := make({}, allocator = {})\n{}', { i(1), i(2), i(3, 'context.temp_allocator'), f(function(args)
			local type = args[2][1]
			local is_slice = false
			local open_i, close_i = string.find(type, '%[.*%]')
			if open_i ~= nil and close_i ~= nil and close_i > open_i then
				local collection_type = vim.trim(string.sub(type, open_i + 1, close_i - 1))
				is_slice = collection_type == ''
			end

			local allocator = ''
			if is_slice then allocator = ', allocator = ' .. vim.trim(args[3][1]) end
			return string.format('defer delete(%s%s)', args[1][1], allocator)
		end, { 1, 2, 3 }) })),
		s('pf', fmt('@(private="{}")', { i(1, 'file') })),
	},
	cs = {
		s('prop', fmt('{} {} {{ get; set; }}', { c(1, { t 'public', t 'private', t 'protected' }), i(0), })),
		s('sprop', fmt('{} static {} {{ get; set; }}', { c(1, { t 'public', t 'private', t 'protected' }), i(0), })),
		s('func', fmt('{} {}({}) {{\n\t{}\n}}', { c(1, { t 'public', t 'private', t 'protected' }), i(2), i(3), i(0) })),
		s('sfunc', fmt('{} static {}({}) {{\n\t{}\n}}', { c(1, { t 'public', t 'private', t 'protected' }), i(2), i(3), i(0) })),
	},
})

-- Highlight groups
-- vim.api.nvim_set_hl(0, "PmenuSel", { bg = "#282C34", fg = "NONE" })
-- vim.api.nvim_set_hl(0, "Pmenu", { fg = "#C5CDD9", bg = "#22252A" })

-- vim.api.nvim_set_hl(0, "CmpItemAbbrDeprecated", { fg = "#7E8294", bg = "NONE", strikethrough = true })
-- vim.api.nvim_set_hl(0, "CmpItemAbbrMatch", { fg = "#82AAFF", bg = "NONE", bold = true })
-- vim.api.nvim_set_hl(0, "CmpItemAbbrMatchFuzzy", { fg = "#82AAFF", bg = "NONE", bold = true })
-- vim.api.nvim_set_hl(0, "CmpItemMenu", { fg = "#C792EA", bg = "NONE", italic = true })
--
-- vim.api.nvim_set_hl(0, "CmpItemKindField", { fg = "#EED8DA", bg = "#B5585F" })
-- vim.api.nvim_set_hl(0, "CmpItemKindProperty", { fg = "#EED8DA", bg = "#B5585F" })
-- vim.api.nvim_set_hl(0, "CmpItemKindEvent", { fg = "#EED8DA", bg = "#B5585F" })
--
-- vim.api.nvim_set_hl(0, "CmpItemKindText", { fg = "#C3E88D", bg = "#9FBD73" })
-- vim.api.nvim_set_hl(0, "CmpItemKindEnum", { fg = "#C3E88D", bg = "#9FBD73" })
-- vim.api.nvim_set_hl(0, "CmpItemKindKeyword", { fg = "#C3E88D", bg = "#9FBD73" })
--
-- vim.api.nvim_set_hl(0, "CmpItemKindConstant", { fg = "#FFE082", bg = "#D4BB6C" })
-- vim.api.nvim_set_hl(0, "CmpItemKindConstructor", { fg = "#FFE082", bg = "#D4BB6C" })
-- vim.api.nvim_set_hl(0, "CmpItemKindReference", { fg = "#FFE082", bg = "#D4BB6C" })
--
-- vim.api.nvim_set_hl(0, "CmpItemKindFunction", { fg = "#EADFF0", bg = "#A377BF" })
-- vim.api.nvim_set_hl(0, "CmpItemKindStruct", { fg = "#EADFF0", bg = "#A377BF" })
-- vim.api.nvim_set_hl(0, "CmpItemKindClass", { fg = "#EADFF0", bg = "#A377BF" })
-- vim.api.nvim_set_hl(0, "CmpItemKindModule", { fg = "#EADFF0", bg = "#A377BF" })
-- vim.api.nvim_set_hl(0, "CmpItemKindOperator", { fg = "#EADFF0", bg = "#A377BF" })
--
-- vim.api.nvim_set_hl(0, "CmpItemKindVariable", { fg = "#C5CDD9", bg = "#7E8294" })
-- vim.api.nvim_set_hl(0, "CmpItemKindFile", { fg = "#C5CDD9", bg = "#7E8294" })
--
-- vim.api.nvim_set_hl(0, "CmpItemKindUnit", { fg = "#F5EBD9", bg = "#D4A959" })
-- vim.api.nvim_set_hl(0, "CmpItemKindSnippet", { fg = "#F5EBD9", bg = "#D4A959" })
-- vim.api.nvim_set_hl(0, "CmpItemKindFolder", { fg = "#F5EBD9", bg = "#D4A959" })
--
-- vim.api.nvim_set_hl(0, "CmpItemKindMethod", { fg = "#DDE5F5", bg = "#6C8ED4" })
-- vim.api.nvim_set_hl(0, "CmpItemKindValue", { fg = "#DDE5F5", bg = "#6C8ED4" })
-- vim.api.nvim_set_hl(0, "CmpItemKindEnumMember", { fg = "#DDE5F5", bg = "#6C8ED4" })
--
-- vim.api.nvim_set_hl(0, "CmpItemKindInterface", { fg = "#D8EEEB", bg = "#58B5A8" })
-- vim.api.nvim_set_hl(0, "CmpItemKindColor", { fg = "#D8EEEB", bg = "#58B5A8" })
-- vim.api.nvim_set_hl(0, "CmpItemKindTypeParameter", { fg = "#D8EEEB", bg = "#58B5A8" })
