local feline = require('feline')
local lsp = require('feline.providers.lsp')
local vi_mode = require('feline.providers.vi_mode')

local function fromhl(hl)
	local result = {}
	local list = vim.api.nvim_get_hl(0, { name = hl })

	local handle_bg_fg = function(results, name, color)
		results[name] = string.format("#%06x", color)
	end

	local handle_cterm = function(results, name, index)
		name = name == 'ctermbg' and 'bg' or 'fg'
		local key = 'terminal_color_' .. index
		results[name] = vim.g[key]
	end
	local actions = {
		['bg'] = handle_bg_fg,
		['fg'] = handle_bg_fg,
		['ctermbg'] = handle_cterm,
		['ctermfg'] = handle_cterm,
	}

	for k, v in pairs(list) do
		if actions[k] ~= nil then
			actions[k](result, k, v)
		end
	end
	return result
end

local function term(num, default)
	local key = 'terminal_color_' .. num
	return vim.g[key] and vim.g[key] or default
end
-- '┃', '█', '', '', '', '', '', '', '●'

local colors = {
	bg = fromhl('StatusLine').bg       or '#2E3440',
	alt = fromhl('CursorLine').bg      or '#475062',
	fg = fromhl('StatusLine').fg       or '#8FBCBB',
	hint = fromhl('DiagnosticHint').bg or '#5E81AC',
	info = fromhl('DiagnosticInfo').bg or '#81A1C1',
	warn = fromhl('DiagnosticWarn').bg or '#EBCB8B',
	err = fromhl('DiagnosticError').bg or '#EC5F67',
	black = term(0, '#434C5E'),
	red = term(1, '#EC5F67'),
	green = term(2, '#8FBCBB'),
	yellow = term(3, '#EBCB8B'),
	blue = term(4, '#5E81AC'),
	magenta = term(5, '#B48EAD'),
	cyan = term(6, '#88C0D0'),
	white = term(7, '#ECEFF4'),
}

local mode_theme = {
	['NORMAL'] = colors.white,
	['OP'] = colors.cyan,
	['INSERT'] = colors.cyan,
	['VISUAL'] = colors.yellow,
	['LINES'] = colors.yellow,
	['BLOCK'] = colors.red,
	['REPLACE'] = colors.magenta,
	['V-REPLACE'] = colors.magenta,
	['SHELL'] = colors.cyan,
	['ENTER'] = colors.magenta,
	['MORE'] = colors.magenta,
	['SELECT'] = colors.red,
	['TERM'] = colors.green,
	['NONE'] = colors.fg,
	['COMMAND'] = colors.blue,
}

local icons = {
	linux = '',
	macos = '',
	windows = '',

	lsp = '',
}

local component = {}

component.vim_mode = {
	provider = function()
		return vi_mode.get_vim_mode()
	end,
	hl = function()
		return {
			fg = 'bg',
			bg = vi_mode.get_mode_color(),
			-- style = 'bold',
			name = 'NeovimModeHLColor',
		}
	end,
	left_sep = 'block',
	right_sep = 'block',
}

component.git_branch = {
	provider = 'git_branch',
	hl = {
		fg = 'fg',
		bg = 'bg',
		style = 'bold',
	},
	left_sep = 'block',
	right_sep = '',
}

component.git_add = {
	provider = 'git_diff_added',
	hl = {
		fg = 'green',
		bg = 'bg',
	},
	left_sep = '',
	right_sep = '',
}

component.git_delete = {
	provider = 'git_diff_removed',
	hl = {
		fg = 'red',
		bg = 'bg',
	},
	left_sep = '',
	right_sep = '',
}

component.git_change = {
	provider = 'git_diff_changed',
	hl = {
		fg = 'magenta',
		bg = 'bg',
	},
	left_sep = '',
	right_sep = '',
}

component.separator = {
	provider = function()
		return '┃'
	end,
	hl = {
		fg = 'magenta',
		bg = 'bg',
	},
	left_sep = 'block',
	right_sep = 'block',
}

component.diagnostic_errors = {
	provider = 'diagnostic_errors',
	hl = {
		fg = 'err',
	},
}

component.diagnostic_warnings = {
	provider = 'diagnostic_warnings',
	hl = {
		fg = 'warn',
	},
}

component.diagnostic_hints = {
	provider = 'diagnostic_hints',
	hl = {
		fg = 'hint',
	},
}

component.diagnostic_info = {
	provider = 'diagnostic_info',
	hl = {
		fg = 'info',
	},
}

component.lsp = {
	provider = function()
		if not rawget(vim, 'lsp') then
			return ''
		end

		if vim.o.columns < 120 then
			return ''
		end

		local clients = vim.lsp.get_active_clients({ bufnr = 0 })
		if #clients ~= 0 then
			-- if clients[1].progress:pop() ~= nil then
			-- 	local spinners = {
			-- 		' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ',
			-- 	}
			-- 	local ms = vim.loop.hrtime() / 1000000
			-- 	local frame = math.floor(ms / 120) % #spinners
			-- 	local content = string.format('%%<%s', spinners[frame + 1])
			-- 	return (content or '') .. 'Lsp Loading...'
			-- else
			-- 	return icons.lsp .. ' ' .. tostring(lsp.lsp_client_names())
			-- end
			return icons.lsp .. ' ' .. tostring(lsp.lsp_client_names())
		end
		return ''
	end,
	hl = function()
		-- local progress = vim.lsp.util.get_progress_messages()[1]
		return {
			-- fg = progress and 'yellow' or 'green',
			style = 'bold',
		}
	end,
	left_sep = 'block',
	right_sep = 'block',
}

component.file_type = {
	provider = {
		name = 'file_type',
		opts = {
			filetype_icon = true,
		},
	},
	hl = {
		fg = 'fg',
		bg = 'bg',
	},
	left_sep = 'block',
	right_sep = 'block',
}

component.scroll_bar = {
	provider = function()
		local chars = { '█', '▇', '▆', '▅', '▄', '▃', '▂', '▁' }
		local line_ratio = vim.api.nvim_win_get_cursor(0)[1] / vim.api.nvim_buf_line_count(0)
		local position = math.floor(line_ratio * 100)
		local percentage = ''

		local icon = tostring(chars[math.max(math.floor(line_ratio * #chars), 1)])
		if position <= 5 then
			percentage = icon .. ' TOP'
		elseif position >= 95 then
			percentage = icon .. ' BOT'
		else
			local position_str = tostring(position)
			percentage =  icon .. ' ' .. (#position_str > 1 and position_str or '0' .. position_str) .. '%%'
		end
		return percentage
	end,
	hl = function()
		local position = math.floor(vim.api.nvim_win_get_cursor(0)[1] / vim.api.nvim_buf_line_count(0) * 100)
		local fg
		local style

		if position <= 5 then
			fg = 'cyan'
			style = 'bold'
		elseif position >= 95 then
			fg = 'red'
			style = 'bold'
		else
			fg = 'magenta'
			style = nil
		end
		return {
			fg = fg,
			style = style,
			bg = 'bg',
		}
	end,
	left_sep = '',
	right_sep = '',
}

local left = {
	{ provider = '', hl = { fg = 'fg', bg = 'bg' } },
	component.file_type,
	component.lsp,
	component.diagnostic_errors,
	component.diagnostic_warnings,
	component.diagnostic_info,
	component.diagnostic_hints,
}
local middle = {}
local right = {
	component.vim_mode,
	component.git_branch,
	component.git_add,
	component.git_delete,
	component.git_change,
	component.separator,
	component.scroll_bar,
}

local components = {
	active = {
		left,
		middle,
		right,
	},
}

feline.setup {
	components = components,
	theme = colors,
	vi_mode_colors = mode_theme,
}

feline.winbar.setup()
feline.statuscolumn.setup()

local update_timer = vim.loop.new_timer()
update_timer:start(0, 100, function()
	vim.schedule(function() vim.cmd(':redrawstatus') end)
end)
