local lazypath = vim.fn.stdpath('data') .. '/lazy/lazy.nvim'
if not vim.loop.fs_stat(lazypath) then
	vim.fn.system({
		'git',
		'clone',
		'--filter=blob:none',
		'https://github.com/folke/lazy.nvim.git',
		'--branch=stable', -- latest stable release
		lazypath,
	})
end
vim.opt.rtp:prepend(lazypath)

local lazy = require('lazy')
local utils = require('utils')
lazy.setup{
	'nvim-lua/plenary.nvim',
	{
		'alexghergh/nvim-tmux-navigation',
		config = function()
			local nvim_tmux_nav = require('nvim-tmux-navigation')
			nvim_tmux_nav.setup {
				disable_when_zoomed = true,
				keybindings = {
					left = "<C-h>",
					down = "<C-j>",
					up = "<C-k>",
					right = "<C-l>",
					last_active = "<C-\\>",
					next = "<C-Space>",
				}
			}
		end,
		lazy = false,
	},
	{ 'freddiehaddad/feline.nvim', lazy = false },

	'nvim-telescope/telescope.nvim',
	'nvim-tree/nvim-web-devicons',
	{
		'ThePrimeagen/harpoon',
		config = function()
			require('harpoon').setup{
				tabline = true,
			}

			local mark = require('harpoon.mark')
			local ui = require('harpoon.ui')

			vim.keymap.set('n', '<leader>a', mark.add_file)
			vim.keymap.set('n', '<leader>z', ui.toggle_quick_menu)

			vim.keymap.set('n', '<M-j>', function() ui.nav_file(1) end)
			vim.keymap.set('n', '<M-k>', function() ui.nav_file(2) end)
			vim.keymap.set('n', '<M-l>', function() ui.nav_file(3) end)
			vim.keymap.set('n', '<M-;>', function() ui.nav_file(4) end)
		end
	},
	{
		'rcarriga/nvim-notify',
		config = function ()
			vim.notify = require('notify')
			vim.notify.setup{
				background_colour = "#000000",
				render = "wrapped-compact",
			}
		end,
		lazy = false,
	},
	{ 'nvim-treesitter/nvim-treesitter', build = ':TSUpdate' },
	'nvim-treesitter/playground',
	{
		'numToStr/Comment.nvim',
		config = function()
			require('Comment').setup{}
		end
	},
	{
		"windwp/nvim-autopairs",
		config = function()
			require('nvim-autopairs').setup{}
		end
	},
	{
		'stevearc/dressing.nvim',
		opts = {},
		config = function()
			require('dressing').setup{
				input = {
					enabled = true,
				},
				select = {
					enabled = true,
				}
			}
		end
	},
	{
		'stevearc/overseer.nvim',
		opts = {},
		config = function()
			local overseer = require('overseer')
			overseer.setup{}
			utils.map('n', '<Leader>c', "<Cmd>OverseerRun<Cr>")
			utils.map('n', '<Leader>x', "<Cmd>OverseerToggle<Cr>")
		end
	},
	{
		'stevearc/oil.nvim',
		opts = {},
		dependencies = { 'nvim-tree/nvim-web-devicons' },
		config = function ()
			local oil = require('oil')
			oil.setup{
				default_file_explorer = true,
			}

			vim.keymap.set('n', '-', oil.open, {})

			local discard_and_close = function ()
				oil.discard_all_changes()
				oil.close()
			end
			vim.keymap.set('n', 'gu', discard_and_close, {})
		end,
	},
	{
		'kevinhwang91/nvim-ufo',
		dependencies = { 'kevinhwang91/promise-async' },
		config = function()
			local ufo = require('ufo')

			vim.keymap.set('n', 'zR', ufo.openAllFolds)
			vim.keymap.set('n', 'zM', ufo.closeAllFolds)

			ufo.setup{}
		end
	},

	{'kepano/flexoki-neovim', name = 'flexoki'},

	-- Git
	{ 'lewis6991/gitsigns.nvim', config = function() require('gitsigns').setup{} end },
	'sindrets/diffview.nvim',

	-- LSP
	'neovim/nvim-lsp',
	'neovim/nvim-lspconfig',
	{ 'L3MON4D3/LuaSnip', version = 'v2.*', build = 'make install_jsregexp', },
	'hrsh7th/nvim-cmp',
	'hrsh7th/cmp-nvim-lsp',
	'hrsh7th/cmp-path',
	'hrsh7th/cmp-nvim-lua',
	'hrsh7th/cmp-cmdline',
	{ 'saadparwaiz1/cmp_luasnip', build = 'make install_jsregexp' },
	{
		'Fildo7525/pretty_hover',
		event = 'LspAttach',
		opts = {}
	},
	{
		'stevearc/conform.nvim',
		event = { 'BufWritePre' },
		cmd = { 'ConformInfo' },
		opts = {
			formatters_by_ft = {
				lua = { 'stylua' },
				go = { 'gofmt' },
			},

			format_on_save = {
				quiet = true,
				timeout_ms = 500,
				lsp_fallback = true,
			},
		},
	}
}

vim.cmd('colorscheme flexoki-dark')

require('plug/feline')
require('plug/telescope')
require('plug/treesitter')
require('plug/lsp')
require('plug/cmp')
